/**
* \file twi.c
* @brief gibt Zugriff auf die I2C Schnittstelle des Mikrocontrollers
*/

#include "twi.h"

volatile uint8_t twi_timeout=0;

/** 
* Initialisiert die I2C Schnittstelle des Mikrocontrollers
*/
void TWI_init(void)						// TWBR soll zwischen 10 und 255 sein
{

#ifdef __AVR_ATmega1281__				// SCL und SDA als Eingang definieren + PullUp aktivieren
	DDRD &= ~((1<<PD0)|(1<<PD1));
	PORTD |= (1<<PD0)|(1<<PD1);
#endif
#ifdef __AVR_ATmega32__
	DDRC &= ~((1<<PC0)|(1<<PC1));
	PORTC |= (1<<PC0)|(1<<PC1);
#endif
#ifdef __AVR_ATmega8__
	DDRC &= ~((1<<PC4)|(1<<PC5));
	PORTC |= (1<<PC4)|(1<<PC5);
#endif

	TWSR = 0; 							// kein prescaler
	//TWSR=(1<<TWPS0);					// prescaler = 4
	TWBR = ((F_CPU/twi_clk)-16)/2;		// scl_clk=F_CPU/(16+2*TWBR*4^prescaler) --> TWBR=(F_CPU/scl_clk-16)/(2*4^prescaler)									 
}

/** 
* Sendet eine START-Kondition mit anschlieendem Senden der Adresse und der Datenrichtung
* @param address 7Bit Adresse des Slave
* @param rw legt Datenrichtung fest (TWIread 1, TWIwrite 0, siehe twi.h)
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t TWI_start(uint8_t address, uint8_t rw)
{
	TWCR=(1<<TWINT)|(1<<TWSTA)|(1<<TWEN);	// sende START-Kondition

	twi_timeout=0;
	while (!(TWCR & (1<<TWINT))){			// w8 dass START bertragen wurden
		#if twi_timeout_on==1
		#if twi_timeout_isr==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>twi_timeout_bound) {return true;}
		#endif		
	};	
	//if ((TWSR & 0xF8)!=TWI_START) && (TWSR & 0xF8)!=TWI_REP_STAR) {ERROR();}
	TWDR=(address<<1)+rw;
	TWCR=(1<<TWINT)|(1<<TWEN);				// sende Slave-adresss + RWcommand
	twi_timeout=0;
	while (!(TWCR & (1<<TWINT))){			// w8 dass Adresse bertragen wurden		
		#if twi_timeout_on==1
		#if twi_timeout_isr==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>twi_timeout_bound) {return true;}
		#endif
	}
	//if ((TWSR & 0xF8)!=TWI_MTX_ADR_ACK) && (TWSR & 0xF8)!=TWI_MRX_ADR_ACK) {ERROR();}
	return false;
}

/** 
* Sendet ein Datenbyte (vorherrige Angabe der Datenrichtung TWIwrite bei START-Kondition, siehe 'TWI_start()' )
* @param data zu sendendes Datenbyte
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t TWI_write(uint8_t data)
{
	TWDR = data;							// data in TWDR-Register laden
	TWCR = (1<<TWINT)|(1<<TWEN);			// starte bertargung von DATA
	twi_timeout=0;
	while (!(TWCR & (1<<TWINT))){};			// w8 dass Daten gesendet
		#if twi_timeout_on==1
		#if twi_timeout_isr==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>twi_timeout_bound) {return true;}
		#endif
	//if ((TWSR & 0xF8)!=TWI_MTX_DATA_ACK) {ERROR();}
	return false;
}

/** 
* Empfngt ein Datenbyte ohne senden eines Acknowledgement (vorherrige Angabe der Datenrichtung TWIread bei START-Kondition, siehe 'TWI_start()' )
* @param erg Pointer auf Speicherbereich in den das empfangene Datenbyte abgelegt werden soll
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t TWI_read_nack(uint8_t *erg)					// Lesen und NACK senden
{
	TWCR = (1<<TWINT)|(1<<TWEN);
	twi_timeout=0;
	while(!(TWCR & (1<<TWINT))){
		#if twi_timeout_on==1
		#if twi_timeout_isr==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>twi_timeout_bound) {return true;}
		#endif	
	};
	//if ((TWSR & 0xF8)!=TWI_MRX_DATA_NACK) {ERROR();}
	*erg=TWDR;
	return false;
}

/** 
* Empfngt ein Datenbyte und Besttigt dieses durch Acknowledgement (vorherrige Angabe der Datenrichtung TWIread bei START-Kondition, siehe 'TWI_start()' )
* @param erg Pointer auf Speicherbereich in den das empfangene Datenbyte abgelegt werden soll
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t TWI_read_ack(uint8_t *erg)					// Lesen und ACK senden
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
	twi_timeout=0;
	while(!(TWCR & (1<<TWINT))){
		#if twi_timeout_on==1
		#if twi_timeout_isr==0
		_delay_ms(1);
		twi_timeout++;
		#endif
		if (twi_timeout>twi_timeout_bound) {return true;}
		#endif
	}
	//if ((TWSR & 0xF8)!=TWI_MRX_DATA_ACK) {ERROR();}
	*erg=TWDR;
	return false;
}

/*
uint8_t TWI_read(uint8_t ack_nack)			// Lesen und ACK/NACK senden
{
	if (ack_nack==1){	//ACK
		TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
		twi_timeout=0;
#if twi_timeout_on==1
		while (!(TWCR & (1<<TWINT))&&(twi_timeout<twi_timeout_bound)){};
#else
		while (!(TWCR & (1<<TWINT))){};
#endif
		//if ((TWSR & 0xF8)!=TWI_MRX_DATA_ACK) {ERROR();}
	}
	else {	//NACK
		TWCR = (1<<TWINT)|(1<<TWEN);
		twi_timeout=0;
#if twi_timeout_on==1
		while (!(TWCR & (1<<TWINT))&&(twi_timeout<twi_timeout_bound)){};
#else
		while (!(TWCR & (1<<TWINT))){};
#endif
		//if ((TWSR & 0xF8)!=TWI_MRX_DATA_NACK) {ERROR();}
	}
	return TWDR;
}
*/

/** 
* Sendet eine STOP-Kondition (Freigabe des Busses)
*/
void TWI_stop(void)
{
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);	// sende STOP-Kondition
}
