/**
* \file twi.h
* @brief Konfigurationsdatei fr die I2C Schnittstelle des Mikrocontrollers
*/

#include <avr/io.h>
#include <util/delay.h>

/**
* definiert Mikrocontroller-Taktfrequenz falls diese nicht im Makefile definiert wurde
*/
#ifndef F_CPU
	#define F_CPU 16000000L           
#endif

/**
* definiert TWI-Taktfrequenz
*/
#ifndef twi_clk
	#define twi_clk 100000L           
#endif

/**
* dient zur Angabe der Datenrichtung 'Lesen' bei twi_start (siehe twi.c)
*/
#define TWIread    1	// als Parameter fr TWI_Start

/**
* dient zur Angabe der Datenrichtung 'Schreiben' bei twi_start (siehe twi.c)
*/
#define TWIwrite   0	// als Parameter fr TWI_Start

/**
* gibt an, ob mit einem Timeout gearbeitet werden soll (=1 ja, =0 nein)
*/
#define twi_timeout_on 1    // mit twi_timeout arbieten = 1, ohen twi_timeout arbeiten = 0	

/**
* gibt an, ob mit bei dem Timeout mit einer Timer Interruptroutine gearbeitet werden soll (=1 ja, =0 nein)
*/
#define twi_timeout_isr 0

/**
* dient als Vergleichswert zur Variablen 'twi_timeout' fr das Arbeiten mit einem Timeout
* <br> Dies sollte abhngig von der jeweiligen Timer-Interruptroutine eingestellt werden und so bemessen sein, dass ca 300ms bis zum auslsen eines Timeout verstreichen.
* <br> (Zeit=300ms='twi_timeout_bound'*'Zeit bis zum auslsen der Timer-Interruptroutine'; Bsp. ist Timer-Interruptroutine auf 100ms eingestellt, so sollte 'twi_timeout_bound' 3 sein).
* <br> Ist 'twi_timeout_isr' = 0, so wird nicht mit einer Interuptroutine gearbeitet und 'twi_timeout_bound' gibt direkt die Zeit in Millisekunden an (zwischen 1..254).
*/
#define twi_timeout_bound    250

#define true       1
#define false      0

/**
* globale Variable zum Zhlen der Timer-Interrupts
*/
extern volatile uint8_t twi_timeout;

extern void TWI_init(void);
extern uint8_t TWI_start(uint8_t address, uint8_t rw);
extern uint8_t TWI_write(uint8_t data);
extern uint8_t TWI_read_nack(uint8_t *erg);
extern uint8_t TWI_read_ack(uint8_t *erg);
extern void TWI_stop(void);

/*
// ***************************************************************************
// TWI State codes 
// von http://www.mikrocontroller.net/topic/87597#new
// ***************************************************************************
// General TWI Master staus codes                      
#define TWI_START					0x08  // START has been transmitted  
#define TWI_REP_START				0x10  // Repeated START has been transmitted
#define TWI_ARB_LOST				0x38  // Arbitration lost

// TWI Master Transmitter staus codes                      
#define TWI_MTX_ADR_ACK				0x18  // SLA+W has been tramsmitted and ACK received
#define TWI_MTX_ADR_NACK			0x20  // SLA+W has been tramsmitted and NACK received 
#define TWI_MTX_DATA_ACK			0x28  // Data byte has been tramsmitted and ACK received
#define TWI_MTX_DATA_NACK			0x30  // Data byte has been tramsmitted and NACK received 

// TWI Master Receiver staus codes  
#define TWI_MRX_ADR_ACK				0x40  // SLA+R has been tramsmitted and ACK received
#define TWI_MRX_ADR_NACK			0x48  // SLA+R has been tramsmitted and NACK received
#define TWI_MRX_DATA_ACK			0x50  // Data byte has been received and ACK tramsmitted
#define TWI_MRX_DATA_NACK			0x58  // Data byte has been received and NACK tramsmitted

// TWI Slave Transmitter staus codes
#define TWI_STX_ADR_ACK				0xA8  // Own SLA+R has been received; ACK has been returned
#define TWI_STX_ADR_ACK_M_ARB_LOST	0xB0  // Arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
#define TWI_STX_DATA_ACK			0xB8  // Data byte in TWDR has been transmitted; ACK has been received
#define TWI_STX_DATA_NACK			0xC0  // Data byte in TWDR has been transmitted; NOT ACK has been received
#define TWI_STX_DATA_ACK_LAST_BYTE	0xC8  // Last data byte in TWDR has been transmitted (TWEA = "0"); ACK has been received

// TWI Slave Receiver staus codes
#define TWI_SRX_ADR_ACK				0x60  // Own SLA+W has been received ACK has been returned
#define TWI_SRX_ADR_ACK_M_ARB_LOST	0x68  // Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
#define TWI_SRX_GEN_ACK				0x70  // General call address has been received; ACK has been returned
#define TWI_SRX_GEN_ACK_M_ARB_LOST	0x78  // Arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_ACK		0x80  // Previously addressed with own SLA+W; data has been received; ACK has been returned
#define TWI_SRX_ADR_DATA_NACK		0x88  // Previously addressed with own SLA+W; data has been received; NOT ACK has been returned
#define TWI_SRX_GEN_DATA_ACK		0x90  // Previously addressed with general call; data has been received; ACK has been returned
#define TWI_SRX_GEN_DATA_NACK		0x98  // Previously addressed with general call; data has been received; NOT ACK has been returned
#define TWI_SRX_STOP_RESTART		0xA0  // A STOP condition or repeated START condition has been received while still addressed as Slave

// TWI Miscellaneous status codes
#define TWI_NO_STATE				0xF8  // No relevant state information available; TWINT = "0"
#define TWI_BUS_ERROR				0x00  // Bus error due to an illegal START or STOP condition
*/

